# ===============================================================================================================
#    SrvrPowerCtrl - a plugin for SqueezeCenter 7.3.x / Squeezebox Server 7.4.x
#    Allows shutdown/restart/suspend/hibernation of your Squeezebox Server
#    hardware via SBS's web interface, your Squeezebox's IR remote
#    or via a SBC / Touch / SqueezePlay.
#
#    Version 20091031.005505
#
#    Copyright (C) 2008, 2009 Gordon Harris
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License,
#    version 2, which should be included with this software.
#
#    Based on Adrian C's (aka tenwiseman) Server Power Control plugin:
#    http://adrianonline.wordpress.com/
#    http://forums.slimdevices.com/showthread.php?t=32674
#    Copyright AdrianOnline.Net Feb 2007. Initially modeled on Max Spicer's Max2.pm
#
#    Also includes code from PeterW's AllQuiet plugin: http://www.tux.org/~peterw/slim/slim7/AllQuiet/
#    AllQuiet copyright (c) 2007 by Peter Watkins (peterw@tux.org)
#
#
#    See a duscussion of this plugin at http://forums.slimdevices.com/showthread.php?t=48521
#
# ===============================================================================================================
#
#    CLI.pm -- helper routines for srvrpowerctrl CLI processing
#

package Plugins::SrvrPowerCtrl::CLI;

use base qw(Slim::Plugin::Base);
use strict;

use Slim::Utils::Strings qw(string);

#Global Variables..
use Plugins::SrvrPowerCtrl::Settings ();
use vars qw(%g);
*g = \%Plugins::SrvrPowerCtrl::Settings::g;

#[srverpowerctrl status]
sub cliGetStatus {
	my ($request) = @_;

	Plugins::SrvrPowerCtrl::Util::logHashElements($request);

	$request->setStatusProcessing();

	$request->addResult('_action', 'status:enabled');
	$request->addResult('_version', "version:$g{nAppVersion}");
	$request->addResult('_path', "path:$g{szAppPath}");
	$request->addResult('_os', "os:$g{szOS}");
	$request->addResult('_distro', "distro:$g{szDistro}");
	$request->addResult('_servermac', "server_mac:$g{szServerMAC}");

	$request->setStatusDone();
}

#[srvrpowerctrl actions]		##reports a list of all the available actions possible..does not include CLI only 'actions' like 'status', 'listblock', etc..only actions that will actually do something..
sub cliGetActions {
	my ($request) = @_;
	my @actions = ();
	my $actionlist;

	$request->setStatusProcessing();

	foreach my $item (@{$g{aActions}}) {
		#if ($item->{menuindex} ge 0) {
			push (@actions, $item->{'action'});

		#}
	}

	#if ( Plugins::SrvrPowerCtrl::Menu::findActionItemNum('setblock') eq -1 ) {
	#	push (@actions, 'clearblock');
	#} else {
	#	push (@actions, 'setblock');
	#}

	$actionlist = join(':', @actions);
	$request->addResult('_action', $actionlist);

	$request->setStatusDone();
}

sub cliSetBlock {
	my ($request) = @_;
	my $client = $request->client();
	my $action = $request->getParam( '_action');
	my $blockowner = $request->getParam('_switchclient');
	my $message = $request->getParam( '_message');

	if ( !defined($blockowner) ) {
		$blockowner = 'viacli';
	}

	$g{log}->debug("Attempting to set block via CLI: caller: $blockowner, message: $message");
	my $blockcode = Plugins::SrvrPowerCtrl::Plugin::blockAction($client, 'set', $blockowner, $message);

	if ($blockcode >= 0) {
		$request->addResult('_blockcode', $blockcode);
		$request->setStatusDone();
		Plugins::SrvrPowerCtrl::Util::DisplayPlayerMessage(undef, $message, 10);
		return;
	} else {
		$g{log}->error( "BadParam: $action");
		$request->addResult('_blockcode', $blockcode);
		$request->setStatusBadParams();
		$request->setStatusDone();
		return;
	}

}

sub cliClearBlock {
	my ($request) = @_;
	my $client = $request->client();
	my $action = $request->getParam( '_action');
	my $blockowner = $request->getParam('_switchclient');
	my $message = $request->getParam( '_message');

	if ( !defined($blockowner) ) {
		$blockowner = 'viacli';
	}

	$g{log}->debug("Attempting to clear block via CLI: caller: $blockowner, message: $message");
	my $blockcode = Plugins::SrvrPowerCtrl::Plugin::blockAction($client, 'clear', $blockowner, $message);
	#A zero result indicates that the last block has been clearned
	if ($blockcode >= 0) {
		$request->addResult('_blockcode', $blockcode);
		$request->setStatusDone();
		Plugins::SrvrPowerCtrl::Util::DisplayPlayerMessage(undef, $message, 10);
		return;
	} else {  #A -1 result indicates error..
		$g{log}->error( "BadParam: $action");
		$request->addResult('_blockcode', $blockcode);
		$request->setStatusBadParams();
		#$request->setStatusDone();
		return;
	}

}

sub cliListBlock {
	my ($request) = @_;
	my $n;

	#this call will clear the array if it's empty..
	if (! Plugins::SrvrPowerCtrl::Block::IsBlocked() ) {
		$request->addResult('_block', 'no_blocks'  );
	} else {
		$n = 0;
		foreach my $element (@{$g{aBlockAction}}) {
			if ( defined($element->{'caller'}) ) {
				$request->addResult('_block' . $n, "$element->{'blockcode'}|$element->{'caller'}|$element->{'reason'}"  );
				$n++;
			}
		}
	}
	$request->setStatusDone();
	return;
}

sub cliGetIdle {
	my ($request) = @_;

	Plugins::SrvrPowerCtrl::Util::logHashElements($request);

	$request->setStatusProcessing();

	my ($nIdle, $nEOD) = Plugins::SrvrPowerCtrl::Watchdog::GetIdleCount();
	
	$request->addResult('_idlecount', "idlecount:$nIdle");
	$request->addResult('_eodcount', "eodcount:$nEOD");

	$request->setStatusDone();
}

sub cliSetIdle {
	my ($request) = @_;

	Plugins::SrvrPowerCtrl::Util::logHashElements($request);

	$request->setStatusProcessing();
	
	my $nNewIdle = $request->getParam( '_message');
	my $nNewEOD = $request->getParam('_switchclient');
	my ($nIdle, $nEOD) = Plugins::SrvrPowerCtrl::Watchdog::SetIdleCount($nNewIdle, $nNewEOD);
	
	$request->addResult('_idlecount', "idlecount:$nIdle");
	$request->addResult('_eodcount', "eodcount:$nEOD");

	$request->setStatusDone();
}




sub cliResetIdle {
	my ($request) = @_;

	Plugins::SrvrPowerCtrl::Util::logHashElements($request);

	$request->setStatusProcessing();

	my ($nIdle, $nEOD) = Plugins::SrvrPowerCtrl::Watchdog::ResetIdleCount();
	
	$request->addResult('_idlecount', "idlecount:$nIdle");
	$request->addResult('_eodcount', "eodcount:$nEOD");

	$request->setStatusDone();
}

# -----------------------------------------------------------------------------
# This is the SC-CLI extension offered by the plugin.  Called by CLI, SBC, etc.

sub pluginCLI {
	my ($request) = @_;
	my $client;
	my $curclient;
	my $item;
	my $action = lc($request->getParam( '_action'));
	my $message = $request->getParam( '_message');
	my $switchclient = $request->getParam('_switchclient');
	my $fromJive = $request->getParam('_fromjive');
	my $timeoutmsg;
	
	Plugins::SrvrPowerCtrl::Util::logHashElements($request->{_params});
	

	$client = $request->client();

	#$g{log}->debug( '      client == ' . $client->name() || 'no client' );
	#$g{log}->debug( '      action == ' . $action || 'no action' );
	#$g{log}->debug( '     message == ' . $message || 'no message' );
	#$g{log}->debug( 'switchclient == ' . $switchclient || 'no switchclient');
	#$g{log}->debug( '    fromJive == ' . $fromJive || 'undef' );

	# If an action is already pending, then this is a request to cancel...
	if ($g{tPendingActionTimer}) {
		return CancelRequest($request, $client);
	}

	# Check that this is a valid request
	if( $request->isNotCommand( [['srvrpowerctrl']])) {
		$g{log}->error( "BadDispatch!");
		$request->setStatusBadDispatch();
		return;
	}

	if ($action eq 'status'){
		#Request is just asking for confirmation that SrvrPowerCtrl is enabled..
		return cliGetStatus($request);
	} elsif ($action eq 'actions'){
		#Request is just asking for a list of the enabled actions..
		return cliGetActions($request);
	} elsif ($action eq 'setblock') {
		#check for block/clear requests..
		return cliSetBlock($request);
	} elsif ($action eq 'clearblock') {
		return cliClearBlock($request);
	} elsif ($action eq 'listblock') {
		return cliListBlock($request);
	} elsif ($action eq 'getidlecount') {
		return cliGetIdle($request);
	} elsif ($action eq 'setidlecount') {
		return cliSetIdle($request);
	} elsif ($action eq 'resetidlecount') {
		return cliResetIdle($request);
	}

	

	# Check for other allowed action items..
	$item = Plugins::SrvrPowerCtrl::Menu::findActionItem($action);
	if (!defined($item)) {
		$g{log}->error( "BadParam: $action");
		$request->addResult('_badparam' . "BadParam: $action"  );
		$request->setStatusBadParams();
		return;
	}

	#OK, this is a valid request..let's proceed..
	$request->setStatusProcessing();


	#if (defined($switchclient) && !defined($fromJive)) {
	if (defined($switchclient)) {
		#Make this a minimum time-out..no opportunity to cancel anyway..
		$item->{cancelwait} = 3;
		if ($switchclient eq 'sleep+hold') {
			#undefine this param...it's not really for this purpose anyway..
			$switchclient = undef;
		} else {
			#Is this a non-Jive CLI request to switch to SN?  If so, then $switchclient is a MAC address..
			#$g{log}->debug( "Switchclient == $switchclient");
			$request->clientid($switchclient);
		}
	}

	#cue up the action..
	if (!Plugins::SrvrPowerCtrl::Plugin::prepareAction($client, $item, 0, $fromJive)) {
		$request->addResult('_action', "$action is blocked.\n");
		$request->setStatusDone();
		return;
	}

	$request->addResult('_action', $action . " in " . $item->{cancelwait} . " seconds.\n");
	$request->setStatusDone();
}


sub CancelRequest {
	my ($request, $client) = @_;
	my $message;

	Plugins::SrvrPowerCtrl::Plugin::cancelAction($client);

	#restore our jive menu..
	#$g{log}->debug("Restoring our jive menu");
	#&addJiveMenu();

	$request->setStatusDone();
	return;
}
