# ===============================================================================================================
#    SrvrPowerCtrl - a plugin for SqueezeCenter 7.3.x / Squeezebox Server 7.4.x
#    Allows shutdown/restart/suspend/hibernation of your Squeezebox Server
#    hardware via SBS's web interface, your Squeezebox's IR remote
#    or via a SBC / Touch / SqueezePlay.
#
#    Version 20091031.005505
#
#    Copyright (C) 2008, 2009 Gordon Harris
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License,
#    version 2, which should be included with this software.
#
#    Based on Adrian C's (aka tenwiseman) Server Power Control plugin:
#    http://adrianonline.wordpress.com/
#    http://forums.slimdevices.com/showthread.php?t=32674
#    Copyright AdrianOnline.Net Feb 2007. Initially modeled on Max Spicer's Max2.pm
#
#    Also includes code from PeterW's AllQuiet plugin: http://www.tux.org/~peterw/slim/slim7/AllQuiet/
#    AllQuiet copyright (c) 2007 by Peter Watkins (peterw@tux.org)
#
#
#    See a duscussion of this plugin at http://forums.slimdevices.com/showthread.php?t=48521
#
# ===============================================================================================================
#
#    Help.pm -- handler functions for SrvrPowerCtrl help pages..
#


package Plugins::SrvrPowerCtrl::Help;


use strict;
use base qw(Slim::Web::Settings);

use Slim::Utils::Prefs;
use Slim::Utils::Validate;
use Slim::Utils::OSDetect;
use File::Spec::Functions qw(:ALL);

use Slim::Utils::Strings qw(string);


# if ($^O eq 'MSWin32') {		## Are we running windows?
if ($^O =~ /^m?s?win/i) {		## Are we running windows?
	eval {
	require Win32;
	import Win32;
	#use Win32::SearchPath;
	} ;
}

#Global Variables..
use vars qw(%g);
*g = \%Plugins::SrvrPowerCtrl::Settings::g;
my $hDistros = ();

my $plugin; # the main plugin class

sub new {
	my $class = shift;
	$plugin   = shift;
	$g{log}->debug("Called!");
	my $urlPrefix = "\\/";
	my $urlRedirectHelp	= 'plugins/SrvrPowerCtrl/html/helpredir.html';
	my $urlMainHelp		= 'plugins/SrvrPowerCtrl/html/help.html';
	my $urlTSHelp		= 'plugins/SrvrPowerCtrl/html/troubleshooting.html';
	my $urlDistroHelp;
	
	#Distros we know how to setup..
	my @aDistros = ('RedHat', 'Debian', 'MacOSX', 'Windows', 'UnSuppOS');


	#Get a hash of our supported distros...
	
	
    #undef $hDistros;
	#Flag the OSs we know how to setup up..
    for (@aDistros) { $hDistros->{$_} = 1 }

	#Plugins::SrvrPowerCtrl::Util::logHashElements($hDistros);
	
	#Provide CSRF protection for the page and set the reference to the page's pre-processing handler..
	$urlRedirectHelp	= Plugins::SrvrPowerCtrl::WebUI::EscapeURL($urlRedirectHelp);
	$urlMainHelp		= Plugins::SrvrPowerCtrl::WebUI::EscapeURL($urlMainHelp);
	$urlTSHelp			= Plugins::SrvrPowerCtrl::WebUI::EscapeURL($urlTSHelp);

	
	#for SBS 7.4 and later..
	if ( $g{nSCVersion} >= 7.4 ) {

		Slim::Web::Pages->addPageFunction($urlRedirectHelp, \&RenderRedirectHelpPage);
		Slim::Web::Pages->addPageFunction($urlMainHelp, \&RenderHelpPage);
		Slim::Web::HTTP::CSRF->protect($urlPrefix . $urlMainHelp);

		Slim::Web::Pages->addPageFunction($urlTSHelp, \&RenderTSHelpPage);
		Slim::Web::HTTP::CSRF->protect($urlPrefix . $urlTSHelp);

		
		foreach my $szDistro (@aDistros) {
			$urlDistroHelp = 'plugins/SrvrPowerCtrl/html/' . $szDistro . '-help.html';
			$urlDistroHelp = Plugins::SrvrPowerCtrl::WebUI::EscapeURL($urlDistroHelp);
			Slim::Web::Pages->addPageFunction($urlDistroHelp, \&RenderDistroHelpPage);
			Slim::Web::HTTP::CSRF->protect($urlPrefix . $urlDistroHelp);
			$g{log}->debug("$urlDistroHelp");

		}
	#for SC 7.3.4 and earlier..	
	} else {

		Slim::Web::HTTP::addPageFunction($urlRedirectHelp, \&RenderRedirectHelpPage);
		Slim::Web::HTTP::addPageFunction($urlMainHelp, \&RenderHelpPage);
		Slim::Web::HTTP::protect($urlPrefix . $urlMainHelp);

		Slim::Web::HTTP::addPageFunction($urlTSHelp, \&RenderTSHelpPage);
		Slim::Web::HTTP::protect($urlPrefix . $urlTSHelp);
		
		foreach my $szDistro (@aDistros) {
			$urlDistroHelp = 'plugins/SrvrPowerCtrl/html/' . $szDistro . '-help.html';
			$urlDistroHelp = Plugins::SrvrPowerCtrl::WebUI::EscapeURL($urlDistroHelp);
			Slim::Web::HTTP::addPageFunction($urlDistroHelp, \&RenderDistroHelpPage);
			Slim::Web::HTTP::protect($urlPrefix . $urlDistroHelp);
			$g{log}->debug("$urlDistroHelp");
		}
	
	}

	$g{log}->debug("Added page functions for:");
	$g{log}->debug("$urlRedirectHelp");
	$g{log}->debug("$urlMainHelp");
	$g{log}->debug("$urlTSHelp");

	
	$class->SUPER::new($plugin);
	
	$g{log}->debug("HelpUI is active..");
}

sub RenderRedirectHelpPage {
	my ($client, $params) = @_;
	my $szOSHelpFile;
	
	$g{log}->debug('Called! client == ' . ( eval{$client->name()} || 'no client' ) . ", param == $params");
#	Plugins::SrvrPowerCtrl::Util::logHashElements($params);


	#Are we running windows?
	if ($g{szOS} eq 'win') {
		$szOSHelpFile = 'plugins/SrvrPowerCtrl/html/Windows-help.html';
	## Various linux distros & OSX..
	} elsif ( ( $g{szOS} eq 'unix' || $g{szOS} eq 'mac' ) && ( $hDistros->{ $g{szDistro} } ) ) {
		$szOSHelpFile = 'plugins/SrvrPowerCtrl/html/' . $g{szDistro} . '-help.html';
	# Unsupported OS or distro..
	} else {
		$szOSHelpFile = 'plugins/SrvrPowerCtrl/html/UnSuppOS-help.html';
	}

	$params->{path} = $szOSHelpFile;
	return RenderDistroHelpPage($client, $params);
}

sub RenderHelpPage {
	my ($client, $params) = @_;
	my $szOSHelpLink;
	my $szSBSUser;
	my $szServerName;
	
	$g{log}->debug('Called! client == ' . ( eval{$client->name()} || 'no client' ) . ", param == $params");
#	Plugins::SrvrPowerCtrl::Util::logHashElements($params);

	#Get the SC/SBS username..(NA for windows & mac)
	if ( $g{nSCVersion} >= 7.4 ) {
		$szSBSUser = 'squeezeboxserver';
		$szServerName = 'Squeezebox Server';
	} else {
		$szSBSUser = 'squeezecenter';
		$szServerName = 'SqueezeCenter';
	}

	#Are we running windows?
	if ($g{szOS} eq 'win') {
		$szOSHelpLink = '<a href="/Default/plugins/SrvrPowerCtrl/html/Windows-help.html" ><h4>' . string('PLUGIN_SRVRPOWERCTRL_WINDOWS_HELP_PAGE') . '</h4></a>' ;
	## Various linux distros & OSX..
	} elsif ( ( $g{szOS} eq 'unix' || $g{szOS} eq 'mac' ) && ( $hDistros->{ $g{szDistro} } ) ) {
		$szOSHelpLink = "<a href=\"/Default/plugins/SrvrPowerCtrl/html/$g{szDistro}-help.html\" ><h4>" . string("PLUGIN_SRVRPOWERCTRL_" . uc($g{szDistro}) . "_HELP_PAGE") . '</h4></a>' ;
	# Unsupported OS or distro..
	} else {
		$szOSHelpLink = "<a href=\"/Default/plugins/SrvrPowerCtrl/html/UnSuppOS-help.html\" ><h4>" . string('PLUGIN_SRVRPOWERCTRL_UNSUPPDISTRO_HELP_PAGE') . '</h4></a>' ;
	}

	$params->{distro} = $g{szDistro};
	$params->{setuplink} = $szOSHelpLink;
	$params->{sbsuser} = $szSBSUser;
	$params->{sbsservername} = $szServerName;	# [% sbsservername %]
	
	Plugins::SrvrPowerCtrl::Util::logHashElements($params);
	return Slim::Web::HTTP::filltemplatefile($params->{path}, $params);
}

sub MyFindInPATH {
    my $file = shift;
    #return $file if -e $file;
    foreach (split ';',$ENV{PATH}) {
        return $_.'\\'.$file if -e $_.'/'.$file;
    }
    return undef;
}



sub RenderDistroHelpPage {
	my ($client, $params) = @_;
	my $szDistro = $g{szDistro};
	my $szWarning;
	my $szCheckFile;
	my $szPluginPath;
	my $szSBSUser;
	my $szServerName;

	$g{log}->debug('Called! client == ' . ( eval{$client->name()} || 'no client' ) . ", param == $params");
	
	$szPluginPath = $g{szAppPath};

	#Get the SC/SBS username..(NA for windows & mac)
	if ( $g{nSCVersion} >= 7.4 ) {
		$szSBSUser = 'squeezeboxserver';
		$szServerName = 'Squeezebox Server';
	} else {
		$szSBSUser = 'squeezecenter';
		$szServerName = 'SqueezeCenter';
	}
	
	#Check to see if we're already enabled by looking for file..
	# Windows..
 	if ($g{szOS} eq 'win') {
		$szCheckFile = "SCPowerTool.exe";
		$szCheckFile = MyFindInPATH($szCheckFile);
		$g{log}->debug("MyFindInPATH found " . ($szCheckFile ? $szCheckFile : 'nada') );
	# Various linux distros..
	} elsif ( $g{szOS} eq 'unix' || $g{szOS} eq 'mac' ) {
		#Path to one of our scripts..if we're already setup..
		$szCheckFile = "/usr/local/sbin/spc-wakeup.sh";
	# Unsupported OS..
	} else {
		#Generic help!
		$szDistro = $g{szOS} . ':' . $g{szDistro};
		$szWarning = sprintf('<H3>' . string('PLUGIN_SRVRPOWERCTRL_UNSUPPDISTRO_HELP_MSG') . '</H3>', $szDistro);											
	}


	#we don't worry about different versions of windows..
	if ($g{szOS} eq 'win') {
		if (-e $szCheckFile) {
			#Already set up..
			$szWarning = '<p>' . string('PLUGIN_SRVRPOWERCTRL_ALREADYSETUPWIN_HELP_MSG') . '</p>';
			$szWarning = sprintf($szWarning, $szCheckFile);
		} else {
			$szWarning = '<p>' . string('PLUGIN_SRVRPOWERCTRL_NEEDSSETUP_HELP_MSG') . '</p>';
		}
	} else {
		#Check to see if the help file being loaded matches our OS's distro..
		my $n = index( $params->{path}, $g{szDistro} );
		$g{log}->debug("index( $params->{path}, $g{szDistro} ) == $n");
	
		if ( index( $params->{path}, $g{szDistro} ) == -1 ) {
			$szDistro = $g{szOS} . ':' . $g{szDistro};
			$szWarning = sprintf('<H3>' . string('PLUGIN_SRVRPOWERCTRL_WRONGDISTRO_HELP_MSG') . '</H3>', $szDistro);
			$szPluginPath = '/path/for/example/only';
		} elsif (-e $szCheckFile) {
			#Already set up..
			$szWarning = '<p>' . string('PLUGIN_SRVRPOWERCTRL_ALREADYSETUP_HELP_MSG') . '</p>';
		} else {
			#Need to set up..
			$szWarning = '<p>' . string('PLUGIN_SRVRPOWERCTRL_NEEDSSETUP_HELP_MSG') . '</p>';
		}
	}

	#$g{log}->debug("$message");
	$params->{distro} = $szDistro;
	$params->{warning} = $szWarning;
	$params->{sbsuser} = $szSBSUser;
	$params->{sbsservername} = $szServerName;	# [% sbsservername %]
	$params->{pluginpath} = $szPluginPath;
	$params->{foundfile} = $szCheckFile;
	
	Plugins::SrvrPowerCtrl::Util::logHashElements($params);
	return Slim::Web::HTTP::filltemplatefile($params->{path}, $params);
}

#Trouble-shooting help page 
sub RenderTSHelpPage {
	my ($client, $params) = @_;
	my $szCacheDir;
	my $szPluginsDir;
	my $szSBSUser;
	my $szServerName;
	
	
	$g{log}->debug('Called! client == ' . ( eval{$client->name()} || 'no client' ) . ", param == $params");
	
	#Get the SC/SBS username..(NA for windows & mac)
	if ( $g{nSCVersion} >= 7.4 ) {
		$szSBSUser = 'squeezeboxserver';
		$szServerName = 'Squeezebox Server';
	} else {
		$szSBSUser = 'squeezecenter';
		$szServerName = 'SqueezeCenter';
	}

	
	$szCacheDir   = Slim::Utils::OSDetect::dirsFor('cache');
	$szPluginsDir = Slim::Utils::OSDetect::dirsFor('plugins');
	
	$params->{distro} = $g{szDistro};
	$params->{macaddress} = $g{szServerMAC};

	$params->{cachedir} = $szCacheDir;
	$params->{pluginsdir}  = $szPluginsDir;
	$params->{sbsuser} = $szSBSUser;
	$params->{sbsservername} = $szServerName;	# [% sbsservername %]

	Plugins::SrvrPowerCtrl::Util::logHashElements($params);
	return Slim::Web::HTTP::filltemplatefile($params->{path}, $params);
}

1;

__END__
