# ===============================================================================================================
#    SrvrPowerCtrl - a plugin for SqueezeCenter 7.3.x / Squeezebox Server 7.4.x
#    Allows shutdown/restart/suspend/hibernation of your Squeezebox Server
#    hardware via SBS's web interface, your Squeezebox's IR remote
#    or via a SBC / Touch / SqueezePlay.
#
#    Version 20091031.005505
#
#    Copyright (C) 2008, 2009 Gordon Harris
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License,
#    version 2, which should be included with this software.
#
#    Based on Adrian C's (aka tenwiseman) Server Power Control plugin:
#    http://adrianonline.wordpress.com/
#    http://forums.slimdevices.com/showthread.php?t=32674
#    Copyright AdrianOnline.Net Feb 2007. Initially modeled on Max Spicer's Max2.pm
#
#    Also includes code from PeterW's AllQuiet plugin: http://www.tux.org/~peterw/slim/slim7/AllQuiet/
#    AllQuiet copyright (c) 2007 by Peter Watkins (peterw@tux.org)
#
#
#    See a duscussion of this plugin at http://forums.slimdevices.com/showthread.php?t=48521
#
# ===============================================================================================================
#
#    Jive.pm -- support for displaying SrvrPowerCtrl menu on SBC's extras
#


package Plugins::SrvrPowerCtrl::Jive;

use base qw(Slim::Plugin::Base);
use strict;


#use Slim::Utils::Strings qw(string);
#use Slim::Player::Client;

#Global Variables..
use Plugins::SrvrPowerCtrl::Settings ();
use vars qw(%g);
*g = \%Plugins::SrvrPowerCtrl::Settings::g;


##Tack our menu onto 'Extras'
sub addJiveMenu {
	my ($class) = @_;

	$g{log}->debug( "Initializing jive menu..");

	my $item;
	my @menuParent;

	my $icon = Plugins::SrvrPowerCtrl::Plugin->_pluginDataFor('icon');
	#my $icon = $class->_pluginDataFor('icon');
	# create a node on the extras menu..
	@menuParent = (
		{	stringToken	=> Plugins::SrvrPowerCtrl::Plugin::getDisplayName(),
			weight		=> 100,
			id			=> 'pluginSrvrPowerCtrlMenu',
			#Check to see if this is still needed for 7.3.x..
			'icon-id'	=> $icon,
			node		=> 'extras',
			'actions'	=> {
				'go' => {
						player => 0,
						cmd => [ 'jivesrvrpowerctrlmenu' ],
				},
			},
			window		=> {	titleStyle => 'album',
								'icon-id'	=> $icon,
			},
		}
	);
	Slim::Control::Jive::registerPluginMenu(\@menuParent, 'extras');

	Slim::Control::Request::addDispatch(['jivesrvrpowerctrlmenu'], [0, 1, 0, \&jiveServerPowerControlMenu]);

}


## Tack our sub-menu items on too (Thanks to bklass for this code..)
sub jiveServerPowerControlMenu {

	my $request = shift;
	my $weight = 10;
	my @jivemenu;

	$g{log}->debug( "Processing jive reqest for action items..");

	# Translate our action item data into something Jive can digest..

	foreach my $item (@{$g{aActions}}) {
		if ($item->{menuindex} ge 0) {
			push (@jivemenu,
				{	text    =>  $item->{'menutext'},
					weight  => $weight,
					actions => { do => { cmd => [ 'srvrpowerctrl', $item->{'action'}, $item->{'message'}, undef, 1 ],	} },
					},);
			$weight += 10;
		}
	}

	$request->addResult('count', scalar(@jivemenu));
	$request->addResult('offset', 0);
	for my $i (0..$#jivemenu) {
		$request->addResultLoop('item_loop', $i, 'text', @jivemenu[$i]->{text});
		$request->addResultLoop('item_loop', $i, 'weight', @jivemenu[$i]->{weight});
		$request->addResultLoop('item_loop', $i, 'actions', @jivemenu[$i]->{actions});
	}

	$request->setStatusDone();
}


1;
