# ===============================================================================================================
#    SrvrPowerCtrl - a plugin for SqueezeCenter 7.3.x / Squeezebox Server 7.4.x
#    Allows shutdown/restart/suspend/hibernation of your Squeezebox Server
#    hardware via SBS's web interface, your Squeezebox's IR remote
#    or via a SBC / Touch / SqueezePlay.
#
#    Version 20091031.005505
#
#    Copyright (C) 2008, 2009 Gordon Harris
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License,
#    version 2, which should be included with this software.
#
#    Based on Adrian C's (aka tenwiseman) Server Power Control plugin:
#    http://adrianonline.wordpress.com/
#    http://forums.slimdevices.com/showthread.php?t=32674
#    Copyright AdrianOnline.Net Feb 2007. Initially modeled on Max Spicer's Max2.pm
#
#    Also includes code from PeterW's AllQuiet plugin: http://www.tux.org/~peterw/slim/slim7/AllQuiet/
#    AllQuiet copyright (c) 2007 by Peter Watkins (peterw@tux.org)
#
#
#    See a duscussion of this plugin at http://forums.slimdevices.com/showthread.php?t=48521
#
# ===============================================================================================================
#
#    Menu.pm -- Menu init, display & action routines..
#


package Plugins::SrvrPowerCtrl::Menu;

use base qw(Slim::Plugin::Base);
use strict;

use Slim::Utils::Strings qw(string);

#Global Variables..
use Plugins::SrvrPowerCtrl::Settings ();
use vars qw(%g);
*g = \%Plugins::SrvrPowerCtrl::Settings::g;

# Initialize the menu array var..
# -------------------------------
sub initActionItems {

	my $menuindex = 0;
	my $szCustCmdLabel;
	my $szCustCmdCommand;

	#$g{log}->debug( "Initializing ActionItems.  Squeezecenter version is $g{nSCVersion}");

	#clear the arrays;
	@{ $g{aActions} } = ();

	push ( @{$g{aActions}},
				{	action					=> 'shutdown',
					actionid				=> 0x00000001,
					menuindex				=> ($g{prefs}->bInclude_Shutdown ? $menuindex++ : -1),
					menutext				=> string('PLUGIN_SRVRPOWERCTRL_SHUTDOWN'),
					message					=> string('PLUGIN_SRVRPOWERCTRL_SHUTDOWN_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_SHUTDOWN_DONE_MSG'),
					dispblock				=> 1,
					#
					#getcommandargcoderef -- executed in Plugins::SrvrPowerCtrl::Plugin::performAction
					#	Args to the coderef will be: ($client $item)
					#	..should return one of the following:
					#	  undef == no cmd line args to feed to the script;
					#	  -1 == error: don't execute the $item->{command};
					#	  anythihng else: is a cmd line arg to pass on to $item->{command}.
					#	For testing, use:
					#getcommandargcoderef	=> \&Plugins::SrvrPowerCtrl::Util::TestCoderefFail,
					#
					getcommandargcoderef	=> undef,
					poweroffplayers			=> $g{prefs}->bPowerOffPlayers,
					gotosn					=> 0,
					checkblock				=> 1,
					cancelwait				=> $g{prefs}->nRegretDelay,
					cmdwait					=> 5,
					refreshwait				=> -1,
					stopsc					=> 1,
					setrtcwakeup			=> $g{prefs}->bSetRTCWakeForAlarm,
					isSleepDefered			=> 0,
					command					=> $g{prefs}->szShutdown_cmd },
	);

	push ( @{$g{aActions}},
				{	action					=> 'restart',
					actionid				=> 0x00000002,
					menuindex				=> ($g{prefs}->bInclude_Restart ? $menuindex++ : -1),
					menutext				=> string('PLUGIN_SRVRPOWERCTRL_RESTART'),
					message					=> string('PLUGIN_SRVRPOWERCTRL_RESTART_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_RESTART_DONE_MSG'),
					dispblock				=> 1,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> 0,
					gotosn					=> 0,
					checkblock				=> 1,
					cancelwait				=> $g{prefs}->nRegretDelay,
					cmdwait					=> 5,
					refreshwait				=> 90,
					stopsc					=> 0,
					setrtcwakeup			=> 0,
					isSleepDefered			=> 0,
					command					=> $g{prefs}->szRestart_cmd },
	);

	push ( @{$g{aActions}},
				{	action					=> 'suspend',
					actionid				=> 0x00000004,
					menuindex				=> ($g{prefs}->bInclude_Suspend ? $menuindex++ : -1),
					menutext				=> string('PLUGIN_SRVRPOWERCTRL_SUSPEND'),
					message					=> string('PLUGIN_SRVRPOWERCTRL_SUSPEND_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_SUSPEND_DONE_MSG'),
					dispblock				=> 1,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> $g{prefs}->bPowerOffPlayers,
					gotosn					=> 0,
					checkblock				=> 1,
					cancelwait				=> $g{prefs}->nRegretDelay,
					cmdwait					=> 5,
					refreshwait				=> -1,
					stopsc					=> 0,
					setrtcwakeup			=> $g{prefs}->bSetRTCWakeForAlarm,
					isSleepDefered			=> 0,
					command					=> $g{prefs}->szSuspend_cmd },
	);

	push ( @{$g{aActions}},
				{	action					=> 'hibernate',
					actionid				=> 0x00000008,
					menuindex				=> ($g{prefs}->bInclude_Hibernate ? $menuindex++ : -1),
					menutext				=> string('PLUGIN_SRVRPOWERCTRL_HIBERNATE'),
					message					=> string('PLUGIN_SRVRPOWERCTRL_HIBERNATE_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_HIBERNATE_DONE_MSG'),
					dispblock				=> 1,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> $g{prefs}->bPowerOffPlayers,
					gotosn					=> 0,
					checkblock				=> 1,
					cancelwait				=> $g{prefs}->nRegretDelay,
					cmdwait					=> 5,
					refreshwait				=> -1,
					stopsc					=> 0,
					setrtcwakeup			=> $g{prefs}->bSetRTCWakeForAlarm,
					isSleepDefered			=> 0,
					command					=> $g{prefs}->szHibernate_cmd },
	);

	push ( @{$g{aActions}},
				{	action					=> 'gotosn',
					actionid				=> 0x00000100,
					menuindex				=> -1,								#Just a CLI action..not on any menus..
					menutext				=> "",
					message					=> string('PLUGIN_SRVRPOWERCTRL_PUSH2SN_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_PUSH2SN_DONE_MSG'),
					dispblock				=> 1,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> 0,
					gotosn					=> 0,
					checkblock				=> 0,
					cancelwait				=> 0,
					cmdwait					=> 1,
					refreshwait				=> 1,
					stopsc					=> 0,
					setrtcwakeup			=> $g{prefs}->bSetRTCWakeForAlarm,
					isSleepDefered			=> 0,
					command					=> \&Plugins::SrvrPowerCtrl::Util::PushToSqueezeNetwork },
	);

	push ( @{$g{aActions}},
				{	action					=> 'getfromsn',
					actionid				=> 0x00001000,
					menuindex				=> -1,								#Just a CLI action..not on any menus..
					menutext				=> "",
					message					=> string('PLUGIN_SRVRPOWERCTRL_GETFROMSN_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_GETFROMSN_DONE_MSG'),
					dispblock				=> 1,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> 0,
					gotosn					=> 0,
					checkblock				=> 0,
					cancelwait				=> 0,
					cmdwait					=> 1,
					refreshwait				=> 1,
					stopsc					=> 0,
					setrtcwakeup			=> 0,
					isSleepDefered			=> 0,
					command					=> \&Plugins::SrvrPowerCtrl::Util::PullFromSqueezeNetwork },
	);

	push ( @{$g{aActions}},
				{	action					=> 'shutdown2sn',
					actionid				=> 0x00000101,
					menuindex				=> ($g{prefs}->bInclude_Shutdown2SN ? $menuindex++ : -1),
					menutext				=> string('PLUGIN_SRVRPOWERCTRL_SHUTDOWN2SN'),
					message					=> string('PLUGIN_SRVRPOWERCTRL_SHUTDOWN2SN_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_SHUTDOWN2SN_DONE_MSG'),
					dispblock				=> 1,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> 0,
					gotosn					=> 1,
					checkblock				=> 1,
					cancelwait				=> $g{prefs}->nRegretDelay,
					cmdwait					=> $g{prefs}->nGoToSNDelay,
					refreshwait				=> $g{prefs}->nGoToSNDelay,
					stopsc					=> 1,
					setrtcwakeup			=> $g{prefs}->bSetRTCWakeForAlarm,
					isSleepDefered			=> 0,
					command					=> $g{prefs}->szShutdown_cmd },
	);

	push ( @{$g{aActions}},
				{	action					=> 'suspend2sn',
					actionid				=> 0x00000104,
					menuindex				=> ($g{prefs}->bInclude_Suspend2SN ? $menuindex++ : -1),
					menutext				=> string('PLUGIN_SRVRPOWERCTRL_SUSPEND2SN'),
					message					=> string('PLUGIN_SRVRPOWERCTRL_SUSPEND2SN_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_SUSPEND2SN_DONE_MSG'),
					dispblock				=> 1,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> 0,
					gotosn					=> 1,
					checkblock				=> 1,
					cancelwait				=> $g{prefs}->nRegretDelay,
					cmdwait					=> $g{prefs}->nGoToSNDelay,
					refreshwait				=> $g{prefs}->nGoToSNDelay,
					stopsc					=> 0,
					setrtcwakeup			=> $g{prefs}->bSetRTCWakeForAlarm,
					isSleepDefered			=> 0,
					command					=> $g{prefs}->szSuspend_cmd },
	);

	push ( @{$g{aActions}},
				{	action					=> 'hibernate2sn',
					actionid				=> 0x00000108,
					menuindex				=> ($g{prefs}->bInclude_Hibernate2SN ? $menuindex++ : -1),
					menutext				=> string('PLUGIN_SRVRPOWERCTRL_HIBERNATE2SN'),
					message					=> string('PLUGIN_SRVRPOWERCTRL_HIBERNATE2SN_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_HIBERNATE2SN_DONE_MSG'),
					dispblock				=> 1,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> 0,
					gotosn					=> 1,
					checkblock				=> 1,
					cancelwait				=> $g{prefs}->nRegretDelay,
					cmdwait					=> $g{prefs}->nGoToSNDelay,
					refreshwait				=> $g{prefs}->nGoToSNDelay,
					stopsc					=> 0,
					setrtcwakeup			=> $g{prefs}->bSetRTCWakeForAlarm,
					isSleepDefered			=> 0,
					command					=> $g{prefs}->szHibernate_cmd },
	);

	push ( @{$g{aActions}},
				{	action					=> 'screstart',
					actionid				=> 0x80000000,
					menuindex				=> ($g{prefs}->bInclude_SCRestart ? $menuindex++ : -1),
					menutext				=> string('PLUGIN_SRVRPOWERCTRL_SCRESTART'),
					message					=> string('PLUGIN_SRVRPOWERCTRL_SCRESTART_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_SCRESTART_DONE_MSG'),
					dispblock				=> 1,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> 0,
					gotosn					=> 0,
					checkblock				=> 1,
					cancelwait				=> $g{prefs}->nRegretDelay,
					cmdwait					=> 5,
					refreshwait				=> 60,
					stopsc					=> 0,	#let the script stop the service!
					setrtcwakeup			=> 0,
					isSleepDefered			=> 0,
#					command					=> 	( $g{nSCVersion} >= 7.4 ? \&Plugins::SrvrPowerCtrl::Util::RestartSC : $g{prefs}->szSCRestart_cmd ) },
					command					=> $g{prefs}->szSCRestart_cmd },
	);


   for ( my $n = 0; $n < 4; $n++ ) {
	  $szCustCmdLabel   = $g{prefs}->{'szCustCmd' . $n . '_lbl'};
	  $szCustCmdCommand = $g{prefs}->{'szCustCmd' . $n . '_cmd'};

	  if ( $szCustCmdLabel && $szCustCmdCommand ) {
		 $g{log}->debug("Adding custom command # $n : $szCustCmdLabel : $szCustCmdCommand ");
		 push ( @{$g{aActions}},
				  {	action					=> "customcmd$n",
					actionid				=> 0x00040000 + $n,
					menuindex				=> $menuindex++,
					menutext				=> $szCustCmdLabel,
					message					=> string('PLUGIN_SRVRPOWERCTRL_CUSTOMCMD_MSG') . ' ' . $szCustCmdLabel,
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_CUSTOMCMD_DONE_MSG'),
					dispblock				=> 1,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> 0,
					gotosn					=> 0,
					checkblock				=> 1,
					cancelwait				=> $g{prefs}->nRegretDelay,
					cmdwait					=> 5,
					refreshwait				=> 60,
					stopsc					=> 0,	
					setrtcwakeup			=> 0,
					isSleepDefered			=> 0,
					command					=> $szCustCmdCommand },
		 );
	  }
   }

	#$g{log}->debug("Adding block-unblock menu item");
	if ( ! Plugins::SrvrPowerCtrl::Block::IsBlocked() ) {
		push ( @{$g{aActions}},
				{	action					=> 'setblock',
					actionid				=> 0x00010000,
					menuindex				=> $menuindex++,
					menutext				=> string('PLUGIN_SRVRPOWERCTRL_SETBLOCK'),
					message					=> string('PLUGIN_SRVRPOWERCTRL_SETBLOCK_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_SETBLOCK_DONE_MSG'),
					dispblock				=> 0,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> 0,
					gotosn					=> 0,
					checkblock				=> 0,
					cancelwait				=> 0,
					cmdwait					=> 2,
					refreshwait				=> 15,
					stopsc					=> 0,
					setrtcwakeup			=> 0,
					isSleepDefered			=> 0,
					command					=> \&Plugins::SrvrPowerCtrl::Block::SetBlock,
				}
		);

	} else {
		push ( @{$g{aActions}},
				{	action					=> 'clearblock',
					actionid				=> 0x00020000,
					menuindex				=> $menuindex++,
					menutext				=> string('PLUGIN_SRVRPOWERCTRL_CLEARBLOCK'),
					message					=> string('PLUGIN_SRVRPOWERCTRL_CLEARBLOCK_MSG'),
					messagedone				=> string('PLUGIN_SRVRPOWERCTRL_CLEARBLOCK_DONE_MSG'),
					dispblock				=> 0,
					getcommandargcoderef	=> undef,
					poweroffplayers			=> 0,
					gotosn					=> 0,
					checkblock				=> 0,
					cancelwait				=> 0,
					cmdwait					=> 2,
					refreshwait				=> 15,
					stopsc					=> 0,
					setrtcwakeup			=> 0,
					isSleepDefered			=> 0,
					command					=> \&Plugins::SrvrPowerCtrl::Block::ClearBlock,
				}
		);
	}

	#Plugins::SrvrPowerCtrl::Util::logHashElements($g{aActions});

}


# Called from CLI..return the whole menuitem
# from the actionitem string..
# --------------------------------------------
sub findActionItem {
	my $szAction = shift;

	if (!defined($szAction)) {
		return undef;
	}

	$szAction = lc($szAction);

	foreach my $actionItem (@{$g{aActions}}) {
		if ($actionItem->{'action'} eq $szAction) {
			return $actionItem;
		}
	}

	return undef;
}

sub findActionItemByMenuIndex {
	my $nIndex = shift;

	foreach my $actionItem (@{$g{aActions}}) {
		if ($actionItem->{'menuindex'} eq $nIndex) {
			return $actionItem;
		}
	}

	return undef;
}

sub findActionItemNum {
	my $szAction = shift;
	my $nItem = 0;

	if (!defined($szAction)) {
		return -1;
	}

	$szAction = lc($szAction);

	foreach my $actionItem (@{$g{aActions}}) {
		if ($actionItem->{'action'} eq $szAction) {
			return $nItem;
		}
		$nItem++;
	}

	return -1;
}

# Player plugin menu IR remote right or left..
# Set in Plugin::setMode
# --------------------------------------------
sub doInputListCallback {
	my ($client, $keypress) = @_;
	my $listIndex;
	my $action;
	my $item;
	my $message;
	my $timeoutmsg;
	my $nDeferTime;

	if ($keypress eq 'right') {

		# carry out actions
		$listIndex = $client->modeParam('listIndex');
		$g{log}->debug("listIndex == $listIndex..");

		$item = findActionItemByMenuIndex($listIndex);

		if (defined($item)) {
			Plugins::SrvrPowerCtrl::Plugin::prepareAction($client, $item, 0);
		} else {
			$g{log}->error("Could not find action item # $listIndex..");
		}

	} elsif ($keypress eq 'left') {
		Slim::Buttons::Common::popModeRight($client);

		if ( $g{tPendingActionTimer} ) {
			#This is a cancel request..
			Plugins::SrvrPowerCtrl::Plugin::cancelAction($client);
		}

	} else {
		$g{log}->warn( "Unhandled callback item = $keypress");
	}
}

# Player plugin menu IR remote up or down..
# Set in Plugin::setMode
# -----------------------------------------
sub doInputListChange {
	my ($client, $item) = @_;
	$g{log}->debug( "$item...");
}

1;
