# ===============================================================================================================
#    SrvrPowerCtrl - a plugin for SqueezeCenter 7.3.x / Squeezebox Server 7.4.x
#    Allows shutdown/restart/suspend/hibernation of your Squeezebox Server
#    hardware via SBS's web interface, your Squeezebox's IR remote
#    or via a SBC / Touch / SqueezePlay.
#
#    Version 20091031.005505
#
#    Copyright (C) 2008, 2009 Gordon Harris
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License,
#    version 2, which should be included with this software.
#
#    Based on Adrian C's (aka tenwiseman) Server Power Control plugin:
#    http://adrianonline.wordpress.com/
#    http://forums.slimdevices.com/showthread.php?t=32674
#    Copyright AdrianOnline.Net Feb 2007. Initially modeled on Max Spicer's Max2.pm
#
#    Also includes code from PeterW's AllQuiet plugin: http://www.tux.org/~peterw/slim/slim7/AllQuiet/
#    AllQuiet copyright (c) 2007 by Peter Watkins (peterw@tux.org)
#
#
#    See a duscussion of this plugin at http://forums.slimdevices.com/showthread.php?t=48521
#
# ===============================================================================================================
#
#    Settings.pm -- Settings & prefs routines..
#

package Plugins::SrvrPowerCtrl::Settings;


use strict;
use base qw(Slim::Web::Settings);

use Slim::Utils::Prefs;
use Slim::Utils::Validate;
use Slim::Utils::OSDetect;
use Plugins::SrvrPowerCtrl::Util;
#use File::Spec::Functions qw(:ALL);
#use File::Basename;


#if ($^O eq 'MSWin32') {		## Are we running windows?
if ($^O =~ /^m?s?win/i) {		## Are we running windows?
	eval {
	require Win32;
	import Win32;
	} ;
}

# Global var hash.  Accessed by all modules in the plugin..

use vars qw(%g);

%g = (
#	nAppVersion				=> $Plugins::SrvrPowerCtrl::Plugin::VERSION,				#version of this code..
	nAppVersion				=> 20091031.005505,											#version of this code..
	szAppPath				=> Plugins::SrvrPowerCtrl::Util::GetPluginPath(),			#Where the heck are we, anyway?
	nSCVersion				=> Plugins::SrvrPowerCtrl::Util::GetSCVersion(),			#Version 20090627.22
	szOS					=> Slim::Utils::OSDetect::OS(),								#Operating system we're running on..
#	szDistro				=> Slim::Utils::OSDetect::getOS()->{osDetails}->{osName},	#OS sub-type..
	szDistro				=> Plugins::SrvrPowerCtrl::Util::GetOSDistro(),				#OS sub-type..
	szServerMAC				=> Plugins::SrvrPowerCtrl::Util::GetServerMacAddress(),		#MAC address of the server..
	aBlockAction			=> [ ],														#Array of block action hashes.. [ ] vs. () is important here to reserve the data space, apparently..
	aActions				=> [ ], 													#Array of action/menu/message/command hash items
	hPendingAction			=> ( ),														#Hash of pending action...
	hPreviousAction			=> ( ),														#Previous action...
	log						=> undef,													#log hash
	prefs					=> undef,													#preferences
	tSPCWatchdogTimer		=> 0,														#our timer ID so we can kill it
	tPendingActionTimer		=> 0,														#timer ID of the pending action...so we can kill it on cancel..
);

#pref names must be in Hungarian notation for generic validation to work.
#pref name prefix == data type and determines type of validation:
#'b' == boolian (i.e. checkbox value)
#'n' == numeric (i.e. int)
#'sz' == string, null terminated

my @prefNames ;

#Hash of pref names and default values.  Some values are OS dependent..

my %prefDefaults = (	'nPrefsVersion'					=>  $g{nAppVersion},

					#Basic Commands---------------------------------------------
						'bInclude_Shutdown'				=>	1,
						'bInclude_Shutdown2SN'			=>	1,
						'szShutdown_cmd'				=>	"",
						'bInclude_Restart'				=>	1,
						'szRestart_cmd'					=>	"",
						'bInclude_Suspend'				=>	1,
						'bInclude_Suspend2SN' 			=>	1,
						'szSuspend_cmd'					=>	"",
						'bInclude_Hibernate'			=>	1,
						'bInclude_Hibernate2SN'			=>	1,
						'szHibernate_cmd'				=>	"",
						'bInclude_SCRestart'			=>	1,
						'szSCRestart_cmd'				=>	"cli://restartserver",
						'bInclude_WebInterface'			=>	1,
					#Custom Commands:-------------------------------------------
						'szCustCmd0_lbl'				=>	"",
						'szCustCmd0_cmd'				=>	"",
						'szCustCmd1_lbl'				=>	"",
						'szCustCmd1_cmd'				=>	"",
						'szCustCmd2_lbl'				=>	"",
						'szCustCmd2_cmd'				=>	"",
						'szCustCmd3_lbl'				=>	"",
						'szCustCmd3_cmd'				=>	"",
						
					#Command Tweaks:--------------------------------------------
						'nRegretDelay'					=>	15,
						'szOnXShutdown_cmd'				=>	"",
						'bPowerOffPlayers'				=>	1,

					#Alternate Server:--------------------------------------------
						'szAltServerLbl'				=>	'mysqueezebox.com',
						'szAltServerAddr'				=>	Slim::Networking::SqueezeNetwork->get_server('sn'),
						'nGoToSNDelay'					=>	60,
						'bNoWebUIRedirect2SN',			=>	0,	#"Manual" pref...not referenced on settings page..
						'bGotoSNAllPlayers'				=>	0,
						'szPushSNPlayerMACs'			=>	"",
						'bOnXShutdownGotoSN'			=>	0,

					#Idle Player Monitor----------------------------------------
						'bUseIdleWatchdog'				=>	0,
						'nIdleWatchdogTimeout'			=>	45,
						'szIdleWatchdogAction'			=>	'suspend',
						'szNotIdleWatchdog_cmd'			=>	"",
					#End of Day-------------------------------------------------
						'bUseEODWatchdog'				=>	0,
						'nEODWatchdogTimeout',			=>	10,
						'szEODWatchdogStartTime'		=>	"00:30",
						'szEODWatchdogEndTime'			=>	"05:00",
						'szEODWatchdogAction'			=>	'shutdown',
						'szEODWatchdog_cmd'				=>	"",
					#Sleep Settings---------------------------------------------
						'bHookSleepButton'				=>	1,
						'szSleepButtonAction'			=>	'plugin_menu',
						'bUseSleepEndWatchdog'			=>	1,
						'szSleepEndWatchdogAction'		=>  'suspend',
					#Alarm Interactions-----------------------------------------
						'bSetRTCWakeForAlarm'			=>	1,
						'nRTCWakeupAdvance'				=>	5,
						'szSetRTCWake_cmd'				=>	"",
					#On-wakeup Actions------------------------------------------
						'bRetrievePlayersFromSN'		=>	1,
						'nRetrievePlayersFromSNWait'	=>	15,
						'szPullSNPlayerMACs'			=>	"",
						'szOnWakeup_cmd'					=>	"",
		     );



#if (!defined( $g{prefs} )) {
#	$g{prefs} = preferences('plugin.srvrpowerctrl');
#	$g{log}->debug("Prefs initialized..");
#}


#my $plugin; # the main plugin class

sub new {
	my $class = shift;
	#my $plugin   = shift;

	#Initialize the log..
	InitLog();

	#Remove any spaces from the distro name

	#$g{szDistro} =~ s/ //g;

	$g{log}->debug("$g{szAppPath}, version: $g{nAppVersion}, running on server: $g{szServerMAC}, OS: $g{szOS}, Distro: $g{szDistro}, SBSVersion: $g{nSCVersion}");

	#Initialize prefs
	InitPrefs();

	$class->SUPER::new;
}


sub InitLog {
	if (!defined( $g{log} )) {
		$g{log} = Slim::Utils::Log->addLogCategory({
			'category'     => 'plugin.SrvrPowerCtrl',
			'defaultLevel' => 'INFO',
			'description'  => 'PLUGIN_SRVRPOWERCTRL_MODULE_NAME',
			});
		$g{log}->debug("Log initialized..");
	}
}

sub InitPrefs {
	$g{log}->debug("Initializing prefs..");

	if (!defined( $g{prefs} )) {
		$g{prefs} = preferences('plugin.srvrpowerctrl');
	} else {
		$g{log}->debug("Prefs already defined!");
	}

	#Report our prefs..
	DispPrefs();

	#Get the pref names from the hash of default values..
	$g{log}->debug("Getting pref names..");
	InitPrefNames();

	#why is this necessary?  Shouldn't the prefs be read automatically?
	$g{log}->debug("Reading prefs..");
	ReadPrefs();

	#Migrate the prefs...this will set any defaults if they don't exist..
	$g{prefs}->migrate(1, \&MigratePrefs);

	#Regularize our prefs..
	$g{log}->debug("Regularizing Prefs..");
	FixUpPrefs();

	#set our prefs change handler..
	$g{log}->debug("Setting Prefs change handler..");
	$g{prefs}->setChange(\&PrefsChange, @prefNames);

	#Report our prefs..
	DispPrefs();

	$g{log}->debug("Prefs initialized..");

	return defined($g{prefs});
}

sub InitPrefNames {
	@prefNames = ( );
	while ( my ($k, $v) = each(%prefDefaults) ) {
		push(@prefNames, $k);
    }
}

sub ReadPrefs {
	foreach my $pref (@prefNames) {
		$g{prefs}->{$pref} = $g{prefs}->get($pref);
	}
}


sub MigratePrefs {

	$g{log}->debug("Migrating prefs..");

	#Check for stale or missing prefs..reset to defaults if necessary..
	#If the plugin version is > than the prefs file version, reset all the prefs to defaults.
	if ( !$g{prefs}->{nPrefsVersion} || $g{prefs}->{nPrefsVersion} < $g{nAppVersion} ) {
		#Don't overwrite prefs that already have values..
		SetPrefDefaults( 0 );
		ReadPrefs();
	}

	$g{log}->debug("Prefs migrated..");
	return 1;
}

#Fixup undefined prefs values to "" or 0
sub FixUpPrefs {
	foreach my $pref (@prefNames) {
		if ( !defined($g{prefs}->$pref) ) {
			#Don't set string prefs to 0
			if ( substr($pref,0,2) eq 'sz' ) {
				$g{prefs}->set($pref, "");
			} else {
				$g{prefs}->set($pref, 0);
			}
		} elsif ( $g{prefs}->$pref eq 'on' ) {
			$g{prefs}->set($pref, 1);
		}
	}
}


sub PrepOSSpecificPrefDefaults {
	my $cmd;
	my $logfile;

	$g{log}->info("Setting default prefs for $g{szOS}, $g{szDistro}");

  	if ($g{szOS} eq 'win') {
		#$cmd = "c:\\windows\\system32\\cmd.exe";
		$cmd = $ENV{windir} . "\\system32\\cmd.exe";
		
		#$cmd =~ s/\\///g;

		#for Windows vista and 7: put the log file somewhere we have write priviliges..
		$logfile =  Slim::Utils::OSDetect::dirsFor('log') . '\\srvrpowerctrl.log';
		#escape any spaces in the logfile path..
		$logfile =~ s/ /\\s/g;

		$prefDefaults{szShutdown_cmd}		=	"$cmd /C start /B scpowertool.exe --shutdown -q \"--log=$logfile\"";
		$prefDefaults{szRestart_cmd}		=	"$cmd /C start /B scpowertool.exe --restart -q \"--log=$logfile\"";
		$prefDefaults{szSuspend_cmd}		=	"$cmd /C start /B scpowertool.exe --standby -q \"--log=$logfile\"";
		$prefDefaults{szHibernate_cmd}		=	"$cmd /C start /B scpowertool.exe --hibernate -q \"--log=$logfile\"";
		$prefDefaults{szSCRestart_cmd}		=	"$cmd /C start /B starthidden.exe scpowertool.exe --screstart -q \"--log=$logfile\"";
		$prefDefaults{szSetRTCWake_cmd}		=	"$cmd /C start /B starthidden.exe scpowertool.exe --wakeup=%d -q \"--log=$logfile\"";
		#Alternate form if using WOSB..
		#$prefDefaults{szSetRTCWake_cmd}	=	"$cmd /C start /B wosb.exe /run /systray /ami dt=%f tm=%t";
		#Deprecated in favor of ReallyPreventStandby plugin..
		#$prefDefaults{szOnWakeup_cmd}		=	"$cmd /C start /B scpowertool.exe --keep-alive=20 -q \"--log=$logfile\"";

	## Various linux distros..
	} elsif ($g{szOS} eq 'unix') {
		$cmd = "sudo";
		$logfile =  Slim::Utils::OSDetect::dirsFor('log') . '/srvrpowerctrl.log';

		$prefDefaults{szShutdown_cmd}		=	"$cmd /sbin/shutdown -h now";
		$prefDefaults{szRestart_cmd}		=	"$cmd /sbin/shutdown -r now";
		$prefDefaults{szSuspend_cmd}		=	"$cmd /usr/sbin/pm-suspend";
		$prefDefaults{szHibernate_cmd}		=	"$cmd /usr/sbin/pm-hibernate";
		$prefDefaults{szSCRestart_cmd}		=	"$cmd /usr/local/sbin/spc-restart.sh";
		#default to HWClock == local time for Debian...
		$prefDefaults{szSetRTCWake_cmd}		=	"$cmd /usr/local/sbin/spc-wakeup.sh " . ($g{szDistro} eq 'Debian' ? "%l" : "%d" ) ;

	## OSX
	} elsif ($g{szOS} eq 'mac') {
		$cmd = "sudo";
		$logfile =  Slim::Utils::OSDetect::dirsFor('log') . '/srvrpowerctrl.log';

		$prefDefaults{szShutdown_cmd}		=	"$cmd /sbin/shutdown -h now";
		$prefDefaults{szRestart_cmd}		=	"$cmd /sbin/shutdown -r now";
		$prefDefaults{szSuspend_cmd}		=	"$cmd /usr/local/sbin/spc-sleep.sh";
		$prefDefaults{szHibernate_cmd}		=	"$cmd /usr/local/sbin/spc-hibernate.sh";
		$prefDefaults{szSCRestart_cmd}		=	"$cmd /usr/local/sbin/spc-restart.sh";
		$prefDefaults{szSetRTCWake_cmd}		=	"$cmd /usr/local/sbin/spc-wakeup.sh %f %t";

	}

	#if we're running on SC 7.4 or above, SCRestart is a cli command..
	if ( $g{nSCVersion} >= 7.4 ) {
		$prefDefaults{szSCRestart_cmd}	= "cli://restartserver";
	}

}

sub SetPrefDefaults {
	my $bForce = shift;
	my $cmd;

	$g{log}->info("Setting prefs to $g{nAppVersion} defaults");
	$g{prefs}->set('nPrefsVersion', $g{nAppVersion} );

	#Fix up the defaults with the OS specific stuff..
	PrepOSSpecificPrefDefaults();

	#Write the default values to the prefs file..
	while ( my ($pref, $default) = each(%prefDefaults) ) {
		#$g{prefs}->set( $pref, $prefDefaults{$pref} );
		if ($bForce) {
			#Don't force overwrite custom commands?
			if ($pref !~ m/^szCustCmd.*/) {
				$g{prefs}->set( $pref, $default );
			}
		}
		#Don't overwrite the prefs if they're defined...
		#elsif ( ! defined($g{prefs}->get($pref)) || ! $g{prefs}->{$pref} ) {
		elsif ( ! defined($g{prefs}->get($pref)) ) {
			$g{prefs}->set( $pref, $default );
		}

    }
}



# Prefs change..
# start using routines form Slim::Utils::Validate.pm to validate values: isInt(), numer(), isTime(), etc.

sub PrefsChange {
	my ($pref, $value) = @_;

	#validate boolian prefs
	if ( substr($pref, 0, 1) eq 'b' ) {
		if (!defined($value)) {
			#fix up the reference to silence a warning in the log..
			@_[1] = 0;
			$value = 0;
			$g{prefs}->{$pref} = $value;
		} elsif ($value eq 'on') {
			@_[1] = 1;
			$value = 1;
			$g{prefs}->{$pref} = $value;
		}
	#validate numeric prefs: if the value is not a number, reset it to the default..
	} elsif ( substr($pref, 0, 1) eq 'n' ) {
		if (!Plugins::SrvrPowerCtrl::Util::IsNumeric($value)) {
			$value = $prefDefaults{$pref};
			@_[1] = $value;
			$g{prefs}->{$pref} = $value;
		}

	#Fixup any string values
	#} elsif ( substr($pref,0,2) eq 'sz' ) {
	#	if (!defined($value)) {
	#		#fix up the reference to silence a warning in the log..
	#		$value = "";
	#	}
	#g{log}->debug("String translation: $g{prefs}->{$pref} vs. $value..");

	#define any other undefined values..
	} else {
		if (!defined($value)) {
			#fix up the reference to silence a warning in the log..
			$value = "";
			@_[1] = $value;
			$g{prefs}->{$pref} = $value;
		}
	}

	#special validations...
	if ( ($pref eq 'szEODWatchdogStartTime') && (!defined(Slim::Utils::Validate::isTime($value))) ) {
		$g{log}->error("$value is not a valid time string!");
		$value = $prefDefaults{$pref};
		@_[1] = $value;
		$g{prefs}->{$pref} = $value;
		$g{prefs}->set($pref, $value);
	}

	if ( ($pref eq 'szEODWatchdogEndTime') && (!defined(Slim::Utils::Validate::isTime($value))) ) {
		$g{log}->error("$value is not a valid time string!");
		$value = $prefDefaults{$pref};
		@_[1] = $value;
		$g{prefs}->{$pref} = $value;
		$g{prefs}->set($pref, $value);
	}

	if ( ($pref =~ m/^szCustCmd/ ) ) {
		$g{log}->error("Paying special attention to $pref");
		$g{prefs}->{$pref} = $value;
		$g{prefs}->set($pref, $value);
	}


	$g{log}->debug("PrefsChange: $pref == $value");

	#Actions to take on prefs change..

	#Basic Commands..
	if ( $pref eq 'bInclude_WebInterface' ) {
		Plugins::SrvrPowerCtrl::WebUI::ActivateWebUI($value);

	#Idle & EOD settings
	} elsif ( $pref eq 'bUseIdleWatchdog' ) {
		Plugins::SrvrPowerCtrl::Watchdog::ActivateIdleWatchdog($value);
	} elsif ( $pref eq 'bUseEODWatchdog' ) {
		Plugins::SrvrPowerCtrl::Watchdog::ActivateEODWatchdog($value);
	} elsif ( $pref eq 'szEODWatchdogStartTime' || $pref eq 'szEODWatchdogEndTime') {
		Plugins::SrvrPowerCtrl::Watchdog::ActivateIdleWatchdog($value);

	#Sleep Settings
	} elsif ( $pref eq 'bHookSleepButton' ) {
		Plugins::SrvrPowerCtrl::SleepButton::HookSleepButton($value);
		Plugins::SrvrPowerCtrl::SleepButton::ChangeSleepHoldButtonHandlers($value);
	} elsif ( $pref eq 'bUseSleepEndWatchdog' ) {
		Plugins::SrvrPowerCtrl::Watchdog::ActivateSleepEndWatchdog($value);

	#Alarm Interactions..
	#} elsif ( $pref eq 'bSetRTCWakeForAlarm' ) {
	#	#Refresh the menu structure...this pref gets pulled into the action items..
	#	Plugins::SrvrPowerCtrl::Menu::initActionItems();

	#On-wakeup Actions..
	} elsif ( $pref eq 'bRetrievePlayersFromSN' ) {
		Plugins::SrvrPowerCtrl::Watchdog::ActivateOnWakeupWatchdog($value);
	} elsif ( $pref eq 'szOnWakeup_cmd' ) {
		Plugins::SrvrPowerCtrl::Watchdog::ActivateOnWakeupWatchdog($value);
	}

	#DispPrefs($pref, "PrefsChange:");

	#reinitialize the action items..
	#if ($pref =~ m/^bInclude_/) {
		#Refresh the menu structure..
		#Plugins::SrvrPowerCtrl::Menu::initActionItems();
	#}

	Plugins::SrvrPowerCtrl::Menu::initActionItems();

	return $value;
}


sub DispPrefs {
	my $showpref = shift;
	my $msg = shift;
	my @pnames;
	my $num = 0;
	my $preflabel;

	if (defined($showpref)) {
		push (@pnames, $showpref);
	} else {
		@pnames = @prefNames;
	}

	if (!defined($msg)) {
		$msg = " ";
	} else {
		$msg = $msg . " ";
	}

	#get the length of the longest prefname
	foreach my $pref (@prefNames) {
		if ($num < length($pref)) {
			$num = length($pref);
		}
	}
	foreach my $pref (@pnames) {
		$preflabel = sprintf('%*2$s', $pref, $num);
		$g{log}->debug("$msg $preflabel == $g{prefs}->{$pref}");
	}
}


#This adds the Name to the Settings->Advanced combobox.
#sub name {
#	return Slim::Web::HTTP::protectName('PLUGIN_SRVRPOWERCTRL_BASIC_SETTINGS');
#}

sub page {
	my $urlSettings = 'plugins/SrvrPowerCtrl/settings/basic.html';
	#my $urlPrefix = "\\/";
	my $urlPrefix = "";

	#$urlSettings = Plugins::SrvrPowerCtrl::WebUI::EscapeURL($urlSettings);

	#for SBS 7.4 and later..
	if ( $g{nSCVersion} >= 7.4 ) {
		#return Slim::Web::HTTP::CSRF->protectURI('plugins/CLI/settings/basic.html');
		return Slim::Web::HTTP::CSRF->protectURI($urlPrefix . $urlSettings);
	} else {
		return Slim::Web::HTTP::protectURI($urlPrefix . $urlSettings);
	}
}

sub prefs {
	return ($g{prefs}, @prefNames );
}

sub handler {
	my ($class, $client, $params) = @_;
	my $action;
	my $item;

	#$g{log}->debug("Settings page handler: params == $params");
	#Plugins::SrvrPowerCtrl::Util::logHashElements($params);

	if ($params->{'reset'}) {
		SetPrefDefaults(1);
		DispPrefs();
	}

	return $class->SUPER::handler($client, $params);
}

sub needsClient {
	return 0;
}

sub getPrefs {
	return $g{prefs};
}



1;

__END__
