#!/bin/sh
# Debian, Ubuntu, etc. setup script for the SrvrPowerCtrl plugin
#

echo 'Setting up Debian-Ubuntu helper scripts for the SrvrPowerCtrl plugin..'

# dash doesn't like [[ $x =~ $y ]] so use this function instead.. 
_index()
{
  case $1 in
    *$2*) 
    idx=${1%%$2*}
    _INDEX=$(( ${#idx} + 1 )) ;;
    *) _INDEX=0; return 1 ;;
  esac
}


#####################################################################################################
#
# Make sure that this script is running under the correct credentials AND from the correct location..
#
#

user=`whoami`
if [ "$user" != "root" ] 
  then
    echo 'Error: This script needs to be run with root credentials,'
    echo "either via # sudo $0"
    echo 'or under su.'
    exit
fi

#Target dir we're copying scripts into..
TRG_SCRIPT_DIR='/usr/local/sbin'

#Change to the source script directory..
REL_DIR=`readlink -f $0`
SRC_SCRIPT_DIR=`dirname "$REL_DIR"`
cd "$SRC_SCRIPT_DIR"
CWD_DIR=`/bin/pwd`

if [ ! "$SRC_SCRIPT_DIR" = "$CWD_DIR" ]
  then
    echo "Error: could not change to directory $SRC_SCRIPT_DIR...exiting!"
    echo "$SRC_SCRIPT_DIR  != $CWD_DIR"
    exit -1
fi

echo "Installing from $SRC_SCRIPT_DIR.."


#####################################################################################################
#
# Establish what we're installing for: SqueezeCenter or Squeezebox Server.  
# Use the appropriate user names.  SBS7.4 squeezeboxserver user is the default.  Fall back to 
# squeezecenter user if 'squeezebox' isn't found in our script path..
#

SBSUSER='squeezeboxserver'
SBSVER='squeezeboxserver'

FOUNDSTR=`echo "$REL_DIR" | /bin/egrep "squeezebox"`
if [ "$FOUNDSTR" = "" ]
  then
    SBSUSER='squeezecenter'
    SBSVER='squeezecenter'
fi

echo "Installing for $SBSVER.."

#####################################################################################################
#
# Fixup permissions on our plugin directory.  
#
# Are we where we think we need to be??
#
# Because the following permissions fix-up uses relative paths, it is potentially 
# dangerious if the script is located in the wrong location, e.g. /home/user.
# So..test that the script is at least located off of a 'squeezecenter' or 
# 'squeezeboxserver' dir.  Also..we don't need to fixup permissions if we've 
# been installed via the Extension Downloader..
#

CUR_DIR=`pwd`
DIR_TEST2='InstalledPlugins'

for DIR_TEST1 in 'squeezecenter' 'squeezeboxserver'
do
	if ( _index "$CUR_DIR" "$DIR_TEST1" )
	  then
	    if  ( ! _index "$CUR_DIR" "$DIR_TEST2" ) 
	      then
	        REL_DIR=`readlink -f ../../.`
	        /bin/echo "Fixing up permissions on $REL_DIR.."
	        #echo "/bin/chown -R $SBSUSER:$SBSUSER ../../."
	        /bin/chown -R "$SBSUSER:nogroup" "$REL_DIR"
			  break
	    fi
	fi
done

#####################################################################################################
#
# Allow the SC/SBS user to use the 'shutdown' command..
#
#ALLOW='/etc/shutdown.allow'
#FOUNDSTR=`/bin/egrep "$SBSUSER" "$ALLOW"`
#if [ "$FOUNDSTR" = "" ]
#  then
#    echo "Adding $SBSUSER to $ALLOW file.."
#    touch "$ALLOW"
#    echo "$SBSUSER" >>"$ALLOW"
#fi

#####################################################################################################
#
# Make modifications to /etc/sudoers...tell sudo that the user has permissions to run
# these commands without raising a password prompt..
#
#

#File we will be modifying..
SUDOERS='/etc/sudoers'
HOST=`/bin/hostname`

MADECHANGES=0

#Make a backup of the sudoers file..
if [ ! -f "$SUDOERS.bak" ]
  then
    echo "Backing up up $SUDOERS to $SUDOERS.bak.."
    cp --force $SUDOERS "$SUDOERS.bak"
  else
    echo "Not backing up $SUDOERS. $SUDOERS.bak already exists.."
fi

#Remove the requiretty directive if present so that the squeezecenter user 
# can run commands and scripts without being logged into a console..

EXPR='^\#Defaults[[:blank:]]*requiretty'
#EXPR='^\#Defaults\s*requiretty'
FOUNDSTR=`/bin/egrep "$EXPR" "$SUDOERS"`

if [ "$FOUNDSTR" = "" ]
  then
    echo "Disabling $SUDOERS requiretty option.."
    sed -i -e 's/\s*Defaults\s*requiretty$/#Defaults    requiretty/' $SUDOERS
    #sed -i -e 's/\s*Defaults *requiretty$/#Defaults    requiretty/' $SUDOERS
    MADECHANGES=1
  else
    echo "Option 'requiretty' already disabled.."
fi

#Tack on permission to run these commands sans password prompt..

#           Shutdown/etc.    Sleep                  Hibernate

for CMD in '/sbin/shutdown*' '/usr/sbin/pm-suspend*' '/usr/sbin/pm-hibernate' '/usr/sbin/pm-powersave*'
do

  #'squeezeboxserver ALL = NOPASSWD: /sbin/shutdown*' 
  EXPR="$SBSUSER.*$CMD"
  FOUNDSTR=`/bin/egrep "$EXPR" "$SUDOERS"`

  if [ "$FOUNDSTR" = "" ]
    then
      echo "Modifying $SUDOERS to allow user $SBSUSER to run $CMD.."
      echo "$SBSUSER ALL = NOPASSWD: $CMD" >>$SUDOERS
      MADECHANGES=1
    else
      echo "User $SBSUSER already has permissions to run $CMD.."
  fi

done

#####################################################################################################
#
# Copy our helper scripts to the target dir, fixup the logpaths and make them executable..
#
#

for scriptnm in spc-*.sh
do 
  if [ ! -e "$TRG_SCRIPT_DIR/$scriptnm" ]
    then
      echo "Installing $scriptnm to $TRG_SCRIPT_DIR"
      cp "$scriptnm" "$TRG_SCRIPT_DIR/$scriptnm"
      sed -i -e "s/SBSVERSION/$SBSVER/g" "$TRG_SCRIPT_DIR/$scriptnm"
      chown root:root "$TRG_SCRIPT_DIR/$scriptnm"
      chmod 755 "$TRG_SCRIPT_DIR/$scriptnm"
  fi

  EXPR="$SBSUSER.*$TRG_SCRIPT_DIR/$scriptnm"
  FOUNDSTR=`/bin/egrep "$EXPR" "$SUDOERS"`

  if [ "$FOUNDSTR" = "" ]
    then
      echo "Modifying $SUDOERS to allow user $SBSUSER to run $TRG_SCRIPT_DIR/$scriptnm.."
      echo "$SBSUSER ALL = NOPASSWD: $TRG_SCRIPT_DIR/$scriptnm*" >>$SUDOERS
      MADECHANGES=1
  else
    echo "User $SBSUSER already has permissions to run $TRG_SCRIPT_DIR/$scriptnm.."
  fi

done



#####################################################################################################
#
# Setup finish..
#

if [ $MADECHANGES = 1 ]
  then
    echo "Done!  Helper scripts installed to $TRG_SCRIPT_DIR and $SUDOERS has been updated."
    exit 0
  else
    echo "No modifications made to $SUDOERS"
    exit 1
fi

