#!/bin/sh
# OSX setup script for the SrvrPowerCtrl plugin
#

user=`whoami`
if [ "$user" != "root" ] 
  then
    echo 'Error: This script needs to be run with root cridentials, via'
    echo "# sudo $0"
    exit
fi

#
if [ "$1" != "" ]
  then
    scuser=$1
  else
    scuser=`/usr/bin/logname`
fi

if [ "$scuser" == "$user" ]
  then
    echo 'Error: user name required!  Run this script'
    echo "via # sudo $0 \$USER"
    exit
fi

if [ "$scuser" == "" ]
  then
    echo 'Error: user name required!  Run this script via:'
    echo "# sudo $0 \$USER"
    exit
fi

echo "Setting up OSX SrvrPowerCtrl helper scripts for user $scuser .."

#change to script directory..
SRC_SCRIPT_DIR=`dirname "$0"`
cd "$SRC_SCRIPT_DIR"
# pwd

TRG_SCRIPT_DIR="/usr/local/sbin"

# Snow leopard: /usr/local may not exist!
if [ ! -d /usr/local ]
  then
    mkdir /usr/local
fi

if [ ! -d /usr/local/sbin ]
  then
    mkdir /usr/local/sbin
fi

#copy the scripts into the scriptdir..
cp ./spc-sleep.sh $TRG_SCRIPT_DIR
cp ./spc-hibernate.sh $TRG_SCRIPT_DIR
cp ./spc-wakeup.sh $TRG_SCRIPT_DIR
cp ./spc-restart.sh $TRG_SCRIPT_DIR


#fixup the log paths in the scripts
sed -i -e "s/usrnm/$scuser/" "$TRG_SCRIPT_DIR/spc-sleep.sh"
sed -i -e "s/usrnm/$scuser/" "$TRG_SCRIPT_DIR/spc-hibernate.sh"
sed -i -e "s/usrnm/$scuser/" "$TRG_SCRIPT_DIR/spc-wakeup.sh"
sed -i -e "s/usrnm/$scuser/" "$TRG_SCRIPT_DIR/spc-restart.sh"


#delete the backups..
rm "$TRG_SCRIPT_DIR/spc-sleep.sh-e"
rm "$TRG_SCRIPT_DIR/spc-hibernate.sh-e"
rm "$TRG_SCRIPT_DIR/spc-wakeup.sh-e"
rm "$TRG_SCRIPT_DIR/spc-restart.sh-e"

#fix permissions on the scripts..
chmod 755 "$TRG_SCRIPT_DIR/spc-sleep.sh"
chown  root:wheel "$TRG_SCRIPT_DIR/spc-sleep.sh"
chmod 755 "$TRG_SCRIPT_DIR/spc-hibernate.sh"
chown  root:wheel "$TRG_SCRIPT_DIR/spc-hibernate.sh"
chmod 755 "$TRG_SCRIPT_DIR/spc-wakeup.sh"
chown  root:wheel "$TRG_SCRIPT_DIR/spc-wakeup.sh"
chmod 755 "$TRG_SCRIPT_DIR/spc-restart.sh"
chown  root:wheel "$TRG_SCRIPT_DIR/spc-restart.sh"


#Allow the squeezecenter user to use the 'shutdown' command..
touch /etc/shutdown.allow
echo "$scuser" >>/etc/shutdown.allow

#Tell sudo that the squeezecenter user has permissions to run these commands..
if [ ! -f /etc/sudoers.bak ]
  then
    cp /etc/sudoers /etc/sudoers.bak
fi


echo "$scuser ALL=NOPASSWD:/sbin/shutdown*" >>/etc/sudoers
echo "$scuser ALL=NOPASSWD:/usr/bin/pmset*" >>/etc/sudoers
echo "$scuser ALL=NOPASSWD:/sbin/SystemStarter*" >>/etc/sudoers
echo "$scuser ALL=NOPASSWD:/usr/local/sbin/spc-sleep.sh" >>/etc/sudoers
echo "$scuser ALL=NOPASSWD:/usr/local/sbin/spc-hibernate.sh" >>/etc/sudoers
echo "$scuser ALL=NOPASSWD:/usr/local/sbin/spc-wakeup.sh*" >>/etc/sudoers
echo "$scuser ALL=NOPASSWD:/usr/local/sbin/spc-restart.sh" >>/etc/sudoers

#OSX has no /var/lock directory for lock files...create one.

if [ ! -d /var/lock ]
  then
    mkdir /var/lock
fi

# Make /var/lock accessible to SrvrPowerCtrl..
chmod 777 /var/lock


echo "Done!  Helper scripts installed to $TRG_SCRIPT_DIR. /etc/sudoers has been updated for user $scuser."
